<?php
/**
 * Plugin Name: WordPress Crypto Payments - Bcon Global
 * Description: Accept Crypto Payments on your WooCommerce-powered website with Bcon Global
 * Version: 1.0.0
 * Author: Bcon Global
 * Author URI: https://bcon.global/
 * Domain Path: /languages/
 * WC tested up to: 6.8.1
 */


if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

require_once ABSPATH . 'wp-admin/includes/plugin.php';
require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
require_once ABSPATH . 'wp-admin/install-helper.php';

/**
 * Initialize hooks needed for the payment gateway
 */
function cprocessing_woocommerce_init()
{
    if (!class_exists('WC_Payment_Gateway')) {
        return;
    }

    require_once plugin_dir_path(__FILE__) . 'php' . DIRECTORY_SEPARATOR . 'WC_Gateway_Cprocessing.php';
    include_once plugin_dir_path(__FILE__) . 'php' . DIRECTORY_SEPARATOR . 'CryptoProcessing.php';

    add_action('admin_menu', 'add_page');
    add_action('init', 'load_plugin_translations');
    add_action('woocommerce_order_details_after_order_table', 'nolo_custom_field_display_cust_order_meta', 10, 1);
    add_action('woocommerce_email_customer_details', 'nolo_cprocessing_woocommerce_email_customer_details', 10, 1);
    add_action('admin_enqueue_scripts', 'cprocessing_load_admin_scripts' );
    add_action('restrict_manage_posts', 'filter_orders' , 20 );
    add_filter('request', 'filter_orders_by_address_or_txid' );
    add_filter('woocommerce_payment_gateways', 'woocommerce_add_cprocessing_gateway');
    add_filter('clean_url', 'cprocessing_async_scripts', 11, 1 );
    /**
     * Add Styles to Bcon Global Admin Page
     **/
    function cprocessing_load_admin_scripts($hook){
        if ( $hook === 'settings_page_cprocessing_options') {
            wp_enqueue_style('cprocessing-admin-style', plugin_dir_url(__FILE__) . "css/cprocessing_options.css", '', get_plugin_data( __FILE__ )['Version']);
        }
    }
    /**
     * Adding new filter to WooCommerce orders
     **/
    function filter_orders() {
		global $typenow;
		if ( 'shop_order' === $typenow ) {
            $filter_by = isset($_GET['filter_by']) ? sanitize_text_field(wp_unslash($_GET['filter_by'])) : "";
			?>
			<input size='26' value="<?php echo($filter_by ); ?>" type='name' placeholder='Filter by crypto address/txid' name='filter_by'>
			<?php
		}
	}
	function filter_orders_by_address_or_txid( $vars ) {
		global $typenow;
		if ( 'shop_order' === $typenow && !empty( $_GET['filter_by'])) {
			$vars['meta_value'] = wc_clean( sanitize_text_field(wp_unslash($_GET['filter_by'])) );
		}
		return $vars;
	}
    /**
     * Add this Gateway to WooCommerce
     **/
    function woocommerce_add_cprocessing_gateway($methods)
    {
        $methods[] = 'WC_Gateway_Cprocessing';
        return $methods;
    }

    function load_plugin_translations()
    {
        load_plugin_textdomain('cprocessing-bitcoin-payments', false, dirname(plugin_basename(__FILE__)) . '/languages/');
    }

    // Add entry in the settings menu
    function add_page()
    {
        $cprocessing = new CryptoProcessing;

        $nonce = isset($_REQUEST['_wpnonce']) ? wp_verify_nonce( sanitize_text_field(wp_unslash($_REQUEST['_wpnonce'])), 'update-options' ) : "";
        if (isset($_POST['generateSecret']) && $nonce)
        {
            generate_secret(true);
        }

        $api_key = $cprocessing->get_api_key();
        // get_api_key() will return api key or temp api key
        // if both are null, generate new cprocessing guest account with temporary wallet
        // temp wallet will be used with temp api key
        if (!$api_key)
        {
            generate_secret();
            $callback_url = get_callback_url();
        }

        add_options_page(
            'CryptoProcessing', 'CryptoProcessing', 'manage_options',
            'cprocessing_options', 'show_options'
        );
    }

    function display_admin_message($msg, $type)
    {
        add_settings_error('option_notice', 'option_notice', $msg, $type);
    }

    function success_message()
    {
        echo '<td colspan="2"class="notice notice-success cprocessing-test-setup-message">'.__("Success", 'cprocessing-bitcoin-payments').'</td>';
    }

    function error_message($error)
    {
        echo
        '<td colspan="2" class="notice notice-error cprocessing-test-setup-message">'.$error.'.<br/>
        Error
        </td>';
    }

    function generate_secret($force_generate = false)
    {
        $callback_secret = get_option("cprocessing_callback_secret");
        if (!$callback_secret || $force_generate) {
            $callback_secret = sha1(openssl_random_pseudo_bytes(20));
            update_option("cprocessing_callback_secret", $callback_secret);
        }
    }

    function get_callback_url()
    {
        $callback_secret = get_option('cprocessing_callback_secret');
        $callback_url = WC()->api_request_url('WC_Gateway_Cprocessing');
        $callback_url = add_query_arg('secret', $callback_secret, $callback_url);
        return $callback_url;
    }

    function show_options()
    {
        if( isset( $_GET[ 'tab' ] ) ) {
            $active_tab = sanitize_key($_GET[ 'tab' ]);
        } else {
            $active_tab = 'settings';
        }
        $settings_updated = isset($_GET['settings-updated']) ? wp_validate_boolean(sanitize_text_field(wp_unslash($_GET['settings-updated']))) : "";
        if ($active_tab == "currencies" && $settings_updated == 'true')
        {
            $cprocessing = new CryptoProcessing;
            $setup_errors = $cprocessing->testSetup();
            $btc_error = isset($setup_errors['btc']) ? $setup_errors['btc'] : 'false';
            $BNB_error = isset($setup_errors['bnb']) ? $setup_errors['bnb'] : 'false';
            $withdraw_requested = $cprocessing->make_withdraw();
        }
        ?>
        <script type="text/javascript">
            function gen_secret() {
                document.generateSecretForm.submit();
            }
            function check_form(tab) {
                const urlParams = new URLSearchParams(window.location.href);
                const currentTab = urlParams.get('tab') ?? 'settings';
                if (currentTab == tab){
                    return;
                }
                if (document.getElementById('cprocessing_form_updated').value == 'true' || document.getElementById('cprocessing_api_updated').value == 'true'){
                    save_form_then_redirect(tab);
                } else {
                    window.location.href = "options-general.php?page=cprocessing_options&tab="+tab;
                }
            }
            function save_form_then_redirect(tab) {
                const xhr = new XMLHttpRequest();
                xhr.open("POST", "options.php");
                xhr.onload = function(event){
                    window.location.href = "options-general.php?page=cprocessing_options&tab="+tab;
                };
                const formData = new FormData(document.myform);
                xhr.send(formData);
                document.getElementById('myform').innerHTML = "Saving Settings...";
            }
            function value_changed() {
                document.getElementById('cprocessing_api_updated').value = 'true';
                add_asterisk("settings");
            }
            function add_asterisk(tab) {
                document.getElementById('cprocessing_form_updated').value = 'true';
                document.getElementById(tab+'_nav_bar').style.background = "#e6d9cb";
                document.getElementById(tab+'_nav_bar').textContent = tab.charAt(0).toUpperCase() + tab.slice(1)+"*";
            }
            function show_advanced() {
                document.getElementById("advanced_title").style.display = 'none';
                document.getElementById("advanced_window").style.display = 'block';
            }
            function show_basic() {
                document.getElementById("advanced_title").style.display = 'block';
                document.getElementById("advanced_window").style.display = 'none';
            }
        </script>
        <script>
            function copy(content) {
                const elem = document.createElement('textarea');
                elem.value = content;
                elem.value = (elem.value).replace(/\s/g, '')
                document.body.appendChild(elem);
                elem.select();
                document.execCommand('copy');
                document.body.removeChild(elem);
                alert("Copied the text: " + content);
            }
        </script>
        <div class="wrap">
            <h1><?php echo __('Bcon Global', 'cprocessing-bitcoin-payments')?></h1>
            <?php
                if (isset($withdraw_requested)):?>
                <div class="cprocessing-width-withdraw">
                    <td colspan='2' class="cprocessing-options-no-padding cprocessing-width">
                        <p class='notice notice-<?php echo $withdraw_requested[1]?>'>
                            <?php echo $withdraw_requested[0].'.' ?>
                        </p>
                    </td>
                </div>
            <?php endif; ?>
            <form method="post" name="myform" id="myform" action="options.php">
                <h2 class="nav-tab-wrapper">
                    <a onclick="check_form('settings')" id='settings_nav_bar'  class="nav-tab <?php echo $active_tab == 'settings' ? 'nav-tab-active' : ''; ?>"><?php echo __('Settings', 'cprocessing-bitcoin-payments')?></a>
                    <a onclick="check_form('currencies')" id='currencies_nav_bar' class="nav-tab <?php echo $active_tab == 'currencies' ? 'nav-tab-active' : ''; ?>"><?php echo __('Currencies', 'cprocessing-bitcoin-payments')?></a>
                </h2>
                <input type="hidden" name="cprocessing_form_updated" id="cprocessing_form_updated" value="false">
                <input type="hidden" name="cprocessing_api_updated" id="cprocessing_api_updated" value="false">
                <?php wp_nonce_field('update-options');
                switch ( $active_tab ){
                case 'settings' :?>
                <div class="cprocessing-width">
                    <h4><?php echo __('Merchant Key BNB', 'cprocessing-bitcoin-payments-btc')?></h4>
                    <input class="cprocessing-options-input-bnb" onchange="value_changed()" size="130" type="text" id="cprocessing_api_key_bnb" name="cprocessing_api_key_bnb" value="<?php echo get_option('cprocessing_api_key_bnb'); ?>" />
                    <h4><?php echo __('Merchant Key BTC', 'cprocessing-bitcoin-payments-bnb')?></h4>
                    <input class="cprocessing-options-input-btc" onchange="value_changed()" size="130" type="text" id="cprocessing_api_key_btc" name="cprocessing_api_key_btc" value="<?php echo get_option('cprocessing_api_key_btc'); ?>" />
                    <h4><?php echo __('Callback URL', 'cprocessing-bitcoin-payments')?>
                        <a href="javascript:gen_secret()" id="generate-callback" class="cprocessing-options-callback-icon" title="Generate New Callback URL">&#xf463;</a>
                    </h4>
                    <input class="cprocessing-options-input" size="130" type="text" value="<?php echo get_callback_url();?>" disabled/> <br>
                    <div class="button-primary" onclick="copy(`<?php echo get_callback_url();?>`)" style="margin-top:10px;">Click to copy</div>
                    <p id="advanced_title" class="cprocessing-options-bold"><a href="javascript:show_advanced()"><?php echo __('Advanced Settings', 'cprocessing-bitcoin-payments')?> &#9660;</a></p>
                    <div id="advanced_window" style="display:none">
                        <p class="cprocessing-options-bold"><a href="javascript:show_basic()"><?php echo __('Advanced Settings', 'cprocessing-bitcoin-payments')?> &#9650;</a></p>
                        <table class="form-table">
                            <tr valign="top">
                                <th scope="row"><?php echo __('Time period of countdown timer on payment page', 'cprocessing-bitcoin-payments')?></th>
                                <td width="70px">
                                    <select onchange="add_asterisk('settings')" name="cprocessing_timeperiod" />
                                        <option value="10" <?php selected(get_option('cprocessing_timeperiod'), 10); ?>>10</option>
                                        <option value="15" <?php selected(get_option('cprocessing_timeperiod'), 15); ?>>15</option>
                                        <option value="20" <?php selected(get_option('cprocessing_timeperiod'), 20); ?>>20</option>
                                        <option value="25" <?php selected(get_option('cprocessing_timeperiod'), 25); ?>>25</option>
                                        <option value="30" <?php selected(get_option('cprocessing_timeperiod'), 30); ?>>30</option>
                                        <option value="60" <?php selected(get_option('cprocessing_timeperiod'), 60); ?>>60</option>
                                    </select>
                                </td>
                                <td>in minutes</td>
                            </tr>
                            <tr valign="top">
                                <th scope="row"><?php echo __('Extra Currency Rate Margin %', 'cprocessing-bitcoin-payments')?></th>
                                <td width="70px"><input onchange="add_asterisk('settings')" type="number" min="0" max="100" step="0.01" name="cprocessing_margin" value="<?php echo esc_attr( get_option('cprocessing_margin', 0) ); ?>" /></td>
                                <td>Increase live fiat to BTC rate by small percent</td>
                            </tr>
                            <tr valign="top">
                                <th scope="row"><?php echo __('Underpayment Slack %', 'cprocessing-bitcoin-payments')?></th>
                                <td width="70px"><input onchange="add_asterisk('settings')" type="number" min="0" max="100" step="0.01" name="cprocessing_underpayment_slack" value="<?php echo esc_attr( get_option('cprocessing_underpayment_slack', 0) ); ?>" /></td>
                                <td>Allow payments that are off by a small percentage</td>
                            </tr>
                            <tr valign="top">
                                <th scope="row"><?php echo __('Display Payment Page in Lite Mode', 'cprocessing-bitcoin-payments')?></th>
                                <td width="70px"><input onchange="add_asterisk('settings')" type="checkbox" name="cprocessing_lite" value="1" <?php checked("1", get_option('cprocessing_lite')); ?> /></td>
                                <td>Enable this if you are having problems in rendering checkout page</td>
                            </tr>
                            <tr valign="top">
                                <th scope="row"><?php echo __('No Javascript checkout page', 'cprocessing-bitcoin-payments')?></th>
                                <td width="70px"><input onchange="add_asterisk('settings')" type="checkbox" name="cprocessing_nojs" value="1" <?php checked("1", get_option('cprocessing_nojs')); ?> /></td>
                                <td>Enable this if you have majority customer that use tor like browser that block Javascript</td>
                            </tr>
                            <tr valign="top">
                                <th scope="row"><?php echo __('Network Confirmations', 'cprocessing-bitcoin-payments')?></th>
                                <td width="70px"><select onchange="add_asterisk('settings')" name="cprocessing_network_confirmation">
                                        <option value="2" <?php selected(get_option('cprocessing_network_confirmation'), 2); ?>><?php echo __('2 (Recommended)', 'cprocessing-bitcoin-payments')?></option>
                                        <option value="1" <?php selected(get_option('cprocessing_network_confirmation'), 1); ?>>1</option>
                                        <option value="0" <?php selected(get_option('cprocessing_network_confirmation'), 0); ?>>0</option>
                                    </select>
                                </td>
                                <td>required for payment to complete</td>
                            </tr>
                        </table>
                    </div>
                    <p class="submit">
                        <input type="submit" class="button-primary" value="<?php echo __("Save", 'cprocessing-bitcoin-payments')?>"/>
                        <input type="hidden" name="action" value="update" />
                        <input type="hidden" name="page_options" value="cprocessing_api_key_bnb,cprocessing_api_key_btc,cprocessing_timeperiod,cprocessing_margin,cprocessing_gen_callback,cprocessing_api_updated,cprocessing_underpayment_slack,cprocessing_lite,cprocessing_nojs,cprocessing_network_confirmation" />
                    </p>
                </form>
                <form method="POST" name="generateSecretForm">
                    <p class="submit">
                        <?php wp_nonce_field('update-options');?>
                        <input type="hidden" name="generateSecret" value="true">
                    </p>
                </form>
                </div>
        <script>
            jQuery('input').on('change', function() {
                if(parseInt(jQuery(this).val()) > parseInt(jQuery(this).attr('max'))) {
                    jQuery(this).val(jQuery(this).attr('max'))
                }
                if(parseInt(jQuery(this).val()) < parseInt(jQuery(this).attr('min'))) {
                    jQuery(this).val(jQuery(this).attr('min'))
                }
            })
        </script>
                    <?php
                    break;
                case 'currencies' :?>
                    <table width="100%" cellspacing="0" cellpadding="0" class="form-table cprocessing-options-intendation cprocessing-width">
                        <h2>
                            <input onchange="add_asterisk('currencies')" type="checkbox" name="cprocessing_btc" value="1"<?php checked("1", get_option('cprocessing_btc', true)); ?>" />
                            <?php echo __('Bitcoin (BTC)', 'cprocessing-bitcoin-payments')?>
                        </h2>
                        <?php
                        $btc_enabled = get_option("cprocessing_btc");
                        if ($btc_enabled || get_option("cprocessing_btc") === false):
                            $total_received = get_option('cprocessing_temp_withdraw_amount') / 1.0e8;
                            $api_key = get_option("cprocessing_api_key_btc");
                        endif;
                        if (get_option('cprocessing_btc') == '1' && isset($btc_error)):
                            if ($btc_error):
                                error_message($btc_error);
                            else:
                                success_message();
                            endif;
                        endif; ?>
                    </table>
                    <table class="form-table cprocessing-options-intendation cprocessing-width">
                        <h2>
                            <input onchange="add_asterisk('currencies')" type="checkbox" name="cprocessing_BNB" value="1"<?php checked("1", get_option('cprocessing_BNB')); ?>" />
                            <?php echo __("Binance Coin (BNB)", 'cprocessing-bitcoin-payments')?>
                        </h2>
                        <?php
                        $BNB_enabled = get_option("cprocessing_BNB");
                        if ($BNB_enabled == '1' && isset($BNB_error)):
                            if ($BNB_error):
                                error_message($BNB_error);
                            else:
                                success_message();
                            endif;
                        endif; ?>
                    </table>
                    <div class="cprocessing-options-small-margin-top">
                        <input type="submit" class="button-primary" value="<?php echo __("Test Setup", 'cprocessing-bitcoin-payments')?>" />
                        <input type="hidden" name="page_options" value="cprocessing_BNB, cprocessing_btc" />
                        <input type="hidden" name="action" value="update" />
                    </div>
                    </form>
                    <?php
                    break;
                }
            ?>
        </div>
    <?php
    }
    function cprocessing_display_tx_info($order, $email=false)
    {
        $cprocessing = new CryptoProcessing();
        $active_cryptos = $cprocessing->getActiveCurrencies();
        foreach ($active_cryptos as $crypto) {
            $txid = get_post_meta($order->get_id(), 'cprocessing_'.$crypto['code'].'_txid', true);
            $address = get_post_meta($order->get_id(), $crypto['code'].'_address', true);
            if ($txid && $address) {
                if ($crypto['code'] == 'btc') {
                    $base_url = CryptoProcessing::BASE_URL;
                }else{
                    $base_url = CryptoProcessing::BNB_BASE_URL;
                }
                echo '<b>'.__('Payment Details', 'cprocessing-bitcoin-payments').'</b><p><strong>'.__('Transaction', 'cprocessing-bitcoin-payments').':</strong>  <a href =\''. $base_url ."/api/tx?txid=$txid&addr=$address'>".substr($txid, 0, 10). '</a></p>';
                if (!$email) {
                   echo '<p>'.__('Your order will be processed on confirmation of above transaction by the bitcoin network.', 'cprocessing-bitcoin-payments').'</p>';
                }
            }
        }
    }
    function nolo_custom_field_display_cust_order_meta($order)
    {
        cprocessing_display_tx_info($order);
    }
    function nolo_cprocessing_woocommerce_email_customer_details($order)
    {
        cprocessing_display_tx_info($order, true);
    }

    function cprocessing_enqueue_stylesheets(){
      wp_enqueue_style('cprocessing-style', plugin_dir_url(__FILE__) . "css/order.css", '', get_plugin_data( __FILE__ )['Version']);
    }

    function cprocessing_enqueue_scripts(){
        wp_enqueue_script( 'reconnecting-websocket', 'https://js.pusher.com/7.2/pusher.min.js', array(), get_plugin_data( __FILE__ )['Version'] );
        wp_enqueue_script( 'qrious', plugins_url('js/vendors/qrious.min.js#deferload', __FILE__), array(), get_plugin_data( __FILE__ )['Version'] );
        wp_enqueue_script( 'cprocessing-checkout', plugins_url('js/checkout.js#deferload', __FILE__), array('reconnecting-websocket', 'qrious'), get_plugin_data( __FILE__ )['Version'] );
    }

    // Async load
    function cprocessing_async_scripts($url)
    {
        if ( strpos( $url, '#deferload') === false )
            return $url;
        else if ( is_admin() )
            return str_replace( '#deferload', '', $url );
        else
        return str_replace( '#deferload', '', $url )."' defer='defer";
    }
}

// After all plugins have been loaded, initialize our payment gateway plugin
add_action('plugins_loaded', 'cprocessing_woocommerce_init', 0);

register_activation_hook( __FILE__, 'cprocessing_activation_hook' );
add_action('admin_notices', 'cprocessing_plugin_activation');

global $cprocessing_db_version;
$cprocessing_db_version = '1.1';

function cprocessing_create_table() {
    // Create cprocessing_orders table
    // https://codex.wordpress.org/Creating_Tables_with_Plugins
    global $wpdb;
    global $cprocessing_db_version;

    $table_name = $wpdb->prefix . 'cprocessing_orders';
    $charset_collate = $wpdb->get_charset_collate();
    $sql = "CREATE TABLE $table_name (
        order_id int NOT NULL,
        status int NOT NULL,
        crypto varchar(3) NOT NULL,
        address varchar(191) NOT NULL,
        satoshi varchar(20) NOT NULL,
        currency varchar(3),
        value longtext,
        txid text,
        memo text,
        KEY orderkey (order_id,crypto)
    ) $charset_collate;";
    dbDelta( $sql );

    update_option( 'cprocessing_db_version', $cprocessing_db_version );
    add_option('cprocessing_timeperiod', 60);
}

function cprocessing_activation_hook() {
    if(!is_plugin_active('woocommerce/woocommerce.php'))
    {
        trigger_error(__( 'Wordpress Crypto Payments - Bcon Global requires WooCommerce plugin to be installed and active.', 'cprocessing-bitcoin-payments' ).'<br>', E_USER_ERROR);
    }

    set_transient( 'cprocessing_activation_hook_transient', true, 3);
}

// Since WP 3.1 the activation function registered with register_activation_hook() is not called when a plugin is updated.
function cprocessing_update_db_check() {
    global $wpdb;
    global $cprocessing_db_version;

    $installed_ver = get_site_option( 'cprocessing_db_version' );
    if ( $installed_ver != $cprocessing_db_version ) {
        $table_name = $wpdb->prefix . 'cprocessing_orders';
        if ($installed_ver < 1.1) {
            maybe_drop_column($table_name, "time_remaining", "ALTER TABLE $table_name DROP COLUMN time_remaining");
            maybe_drop_column($table_name, "timestamp", "ALTER TABLE $table_name DROP COLUMN timestamp");
        }
        cprocessing_create_table();
    }
}

add_action( 'plugins_loaded', 'cprocessing_update_db_check' );
register_activation_hook( __FILE__, 'cprocessing_create_table' );

//Show message when plugin is activated
function cprocessing_plugin_activation() {
  if(!is_plugin_active('woocommerce/woocommerce.php'))
  {
      $html = '<div class="error">';
      $html .= '<p>';
      $html .= __( 'Wordpress Crypto Payments - Bcon Global failed to load. Please activate WooCommerce plugin.', 'cprocessing-bitcoin-payments' );
      $html .= '</p>';
      $html .= '</div>';
      echo $html;
  }
  if( get_transient( 'cprocessing_activation_hook_transient' ) ){

    $html = '<div class="updated">';
    $html .= '<p>';
    $html .= __( 'Congrats, you are now accepting Crypto payments! You can configure Bcon Global <a href="options-general.php?page=cprocessing_options">on this page</a>.', 'cprocessing-bitcoin-payments' );
    $html .= '</p>';
    $html .= '</div>';

    echo $html;
    delete_transient( 'fx-admin-notice-example' );
  }
}

// On uninstallation, clear every option the plugin has set
register_uninstall_hook( __FILE__, 'cprocessing_uninstall_hook' );
function cprocessing_uninstall_hook() {
    delete_option('cprocessing_callback_secret');
    delete_option('cprocessing_api_key_btc');
    delete_option('cprocessing_api_key_bnb');
    delete_option('cprocessing_temp_api_key');
    delete_option('cprocessing_temp_withdraw_amount');
    delete_option('cprocessing_margin');
    delete_option('cprocessing_timeperiod');
    delete_option('cprocessing_api_updated');
    delete_option('cprocessing_BNB');
    delete_option('cprocessing_btc');
    delete_option('cprocessing_underpayment_slack');
    delete_option('cprocessing_lite');
    delete_option('cprocessing_nojs');
    delete_option('cprocessing_network_confirmation');

    global $wpdb;
    $wpdb->query($wpdb->prepare("DROP TABLE IF EXISTS ".$wpdb->prefix."cprocessing_orders"));
    delete_option("cprocessing_db_version");
}


function cprocessing_plugin_add_settings_link( $links ) {
    $settings_link = '<a href="options-general.php?page=cprocessing_options">' . __( 'Settings' ) . '</a>';
    array_unshift( $links, $settings_link );
    return $links;
}
$plugin = plugin_basename( __FILE__ );
add_filter( "plugin_action_links_$plugin", 'cprocessing_plugin_add_settings_link' );

add_action( 'before_woocommerce_init', function() {
    if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
    }
} );