<?php

/**
 * This class is responsible for communicating with the CryptoProcessing API
 */
class CryptoProcessing
{
    const BASE_URL = 'https://external-api.bcon.global';
    const NEW_ADDRESS_URL = 'https://external-api.bcon.global/api/v1/address'; /* POST */
    const PRICE_URL = 'https://external-api.bcon.global/api/v1/currencies/btc'; /* GET */
    const SET_CALLBACK_URL = 'https://external-api.bcon.global/api/v1/stores/set_callback'; /* Patch params: callback_url */
    const GET_CALLBACKS_URL = 'https://external-api.bcon.global/api/v1/stores/get_store';
    const TEMP_API_KEY_URL = '';
    const TEMP_WITHDRAW_URL = '';

    const BNB_BASE_URL = 'https://external-api.bcon.global';
    const BNB_NEW_ADDRESS_URL = 'https://external-api.bcon.global/api/v1/address';
    const BNB_PRICE_URL = 'https://external-api.bcon.global/api/v1/currencies/bnb';
    const BNB_SET_CALLBACK_URL = 'https://external-api.bcon.global/api/v1/stores/set_callback';
    const BNB_GET_CALLBACKS_URL = 'https://external-api.bcon.global/api/v1/stores/get_store';

    protected string $api_key;

    public function __construct()
    {
        $crypto = isset($_GET["crypto"]) ? sanitize_key($_GET['crypto']) : 'btc';
        $this->api_key = $this->get_api_key($crypto);
    }

    public function get_api_key($crypto = 'btc')
    {
        $api_key = get_option("cprocessing_api_key_" . $crypto);
        return $api_key;
    }

    public function test_new_address_gen($crypto, $response)
    {
        $error_str = '';
        $callback_secret = get_option('cprocessing_callback_secret');
        $response = $this->new_address($callback_secret, $crypto, true);
        if ($response->response_code!=200){
            $error_str = $response->response_message;
        }
        return $error_str;
    }

    public function new_address($crypto, $reset = false)
    {
        if ($crypto === 'btc') {
            $url = CryptoProcessing::NEW_ADDRESS_URL;
        } else {
            $url = CryptoProcessing::BNB_NEW_ADDRESS_URL;
        }
        $response = $this->post($url, $this->api_key, '', 8);
        if (!isset($responseObj)) $responseObj = new stdClass();
        $responseObj->{'response_code'} = wp_remote_retrieve_response_code($response);
        if (wp_remote_retrieve_body($response)) {
            $body = json_decode(wp_remote_retrieve_body($response));
            $responseObj->{'response_message'} = isset($body->data->message) ? $body->data->message : '';
            $responseObj->{'address'} = isset($body->data->address) ? $body->data->address : '';
            $responseObj->{'memo'} = isset($body->data->memo) ? $body->data->memo : '';

            if(!empty($responseObj->{'memo'})) {
                $responseObj->{'address'} .= '_' . $responseObj->{'memo'};
            }
        }
        return $responseObj;
    }

    public function get_price($currency, $crypto)
    {
        if ($crypto === 'btc') {
            $url = CryptoProcessing::PRICE_URL . "?currency=$currency";
        } else {
            $url = CryptoProcessing::BNB_PRICE_URL . "?currency=$currency";
        }
        $response = $this->get($url, $this->api_key);

        if (!isset($responseObj)) $responseObj = new stdClass();
        $responseObj->{'response_code'} = wp_remote_retrieve_response_code($response);
        if (wp_remote_retrieve_body($response)) {
            $body = json_decode(wp_remote_retrieve_body($response));
            $responseObj->{'response_message'} = isset($body->data->message) ? $body->data->message : '';
            $responseObj->{'price'} = isset($body->data->price) ? $body->data->price : '';
        }
        return $responseObj;
    }

    public function update_callback($callback_url, $crypto, $xpub)
    {
        if ($crypto === 'btc') {
            $url = CryptoProcessing::SET_CALLBACK_URL;
        } else {
            $url = CryptoProcessing::BNB_SET_CALLBACK_URL;
        }
        $body = [
            'callback_url' => $callback_url,
            'xpub' => $xpub
        ];

        $response = $this->post($url, $this->api_key, $body);

        $responseObj = json_decode(wp_remote_retrieve_body($response));
        if (!isset($responseObj)) $responseObj = new stdClass();
        $responseObj->{'response_code'} = wp_remote_retrieve_response_code($response);
        return $responseObj;
    }

    public function get_callbacks($crypto)
    {
        $this->api_key = $this->get_api_key($crypto);
        if ($crypto === 'btc') {
            $url = CryptoProcessing::GET_CALLBACKS_URL;
        } else {
            $url = CryptoProcessing::BNB_GET_CALLBACKS_URL;
        }
        $response = $this->get($url, $this->api_key);
        return $response;
    }

    public function check_get_callbacks_response_code($response)
    {
        $error_str = '';
        //TODO: Check This: WE should actually check code for timeout
        if (!wp_remote_retrieve_response_code($response)) {
            $error_str = __('Your server is blocking outgoing HTTPS calls', 'cprocessing-bitcoin-payments');
        } elseif (wp_remote_retrieve_response_code($response) == 401)
            $error_str = __('API Key is incorrect', 'cprocessing-bitcoin-payments');
        elseif (wp_remote_retrieve_response_code($response) != 200)
            $error_str = $response->data;
        return $error_str;
    }

    public function check_get_callbacks_response_body($response, $crypto)
    {
        $error_str = '';
        $response_body = json_decode(wp_remote_retrieve_body($response));
        //if merchant doesn't have any xPubs on his CryptoProcessing account
        if (!isset($response_body) || empty($response_body)) {
            $error_str = __('Please add a new store on cprocessing website', 'cprocessing-bitcoin-payments');
        } //if merchant has at least one xPub on his CryptoProcessing account
        elseif (!empty($response_body)) {
            $error_str = $this->examine_server_callback_urls($response_body, $crypto);
        }
        return $error_str;
    }

    // checks each existing xpub callback URL to update and/or use
    public function examine_server_callback_urls($response_body, $crypto)
    {
        $callback_secret = get_option('cprocessing_callback_secret');
        $api_url = WC()->api_request_url('WC_Gateway_Cprocessing');
        $wordpress_callback_url = add_query_arg('secret', $callback_secret, $api_url);
        $base_url = preg_replace('/https?:\/\//', '', $api_url);
        $available_xpub = '';
        $partial_match = '';
        //Go through all xpubs on the server and examine their callback url
        foreach ($response_body->data as $one_response) {
            $server_callback_url = isset($one_response->callback_url) ? $one_response->callback_url : '';
            $server_base_url = preg_replace('/https?:\/\//', '', $server_callback_url);
            $xpub = isset($one_response->xpub) ? $one_response->xpub : '';
            if (!$server_callback_url) {
                // No callback
                $available_xpub = $xpub;
            } else if ($server_callback_url == $wordpress_callback_url) {
                // Exact match
                return '';
            } else if (strpos($server_base_url, $base_url) === false) {
                // Partial Match - Only secret or protocol differ
                $partial_match = $xpub;
            }
        }
        // Use the available xpub
        if ($partial_match || $available_xpub) {
            $update_xpub = $partial_match ? $partial_match : $available_xpub;
            $response = $this->update_callback($wordpress_callback_url, $crypto, $update_xpub);
            if ($response->response_code != 200) {
                return $response->message;
            }
            return '';
        }
        // No match and no empty callback
        $error_str = __("Please add a new store on Bcon Global website", 'cprocessing-bitcoin-payments');
        return $error_str;
    }


    public function check_callback_urls_or_set_one($crypto, $response)
    {
        $api_key = get_option("cprocessing_api_key_" . strtolower($crypto));
        //If BNB enabled and API Key is not set: give error
        if (!$api_key && $crypto === 'bnb') {
            $error_str = __('Set the API Key or disable BNB', 'cprocessing-bitcoin-payments');
            return $error_str;
        }
        //chek the current callback and detect any potential errors
        $error_str = $this->check_get_callbacks_response_code($response, $crypto);
        if (!$error_str) {
            //if needed, set the callback.
            $error_str = $this->check_get_callbacks_response_body($response, $crypto);
        }
        return $error_str;
    }


    public function get_temp_api_key($callback_url)
    {

        $url = CryptoProcessing::TEMP_API_KEY_URL;
        $body = json_encode(array('callback' => $callback_url));
        $response = $this->post($url, '', $body);
        $responseObj = json_decode(wp_remote_retrieve_body($response));
        if(!empty($response)) {
            $responseObj->{'response_code'} = wp_remote_retrieve_response_code($response);
        } else {
            $responseObj = new \stdClass();
            $responseObj->response_code = 500;
        }
        return $responseObj;
    }

    /*
     * Get list of crypto currencies supported by CryptoProcessing
     */
    public function getSupportedCurrencies()
    {
        return array(
            'btc' => array(
                'code' => 'btc',
                'name' => 'Bitcoin',
                'uri' => 'bitcoin'
            ),
            'bnb' => array(
                'code' => 'bnb',
                'name' => 'Binance Coin',
                'uri' => 'binancesmartchain'
            )
        );
    }

    /*
     * Get list of active crypto currencies
     */
    public function getActiveCurrencies()
    {
        $active_currencies = array();
        $cprocessing_currencies = $this->getSupportedCurrencies();
        foreach ($cprocessing_currencies as $code => $currency) {
            $enabled = get_option('cprocessing_' . $code);
            if ($enabled || ($code === 'btc' && $enabled === false)) {
                $active_currencies[$code] = $currency;
            }
        }
        return $active_currencies;
    }

    public function make_withdraw()
    {
        $api_key = $this->api_key;
        $temp_api_key = get_option('cprocessing_temp_api_key');
        if (!$api_key || !$temp_api_key || $temp_api_key == $api_key) {
            return null;
        }
        if (get_option('cprocessing_temp_withdraw_amount') > 0) {

            $url = CryptoProcessing::TEMP_WITHDRAW_URL . '?tempkey=' . $temp_api_key;
            $response = $this->post($url, $api_key);
            $responseObj = json_decode(wp_remote_retrieve_body($response));
            $response_code = wp_remote_retrieve_response_code($response);
            if ($response_code != 200) {
                $message = __('Error while making withdraw: ' . $responseObj->message, 'cprocessing-bitcoin-payments');
                return [$message, 'error'];
            }
            update_option("cprocessing_temp_api_key", null);
            update_option('cprocessing_temp_withdraw_amount', 0);
            $message = __('Your funds withdraw request has been submitted. Please check your Bcon Global registered emailid for details', 'cprocessing-bitcoin-payments');
            return [$message, 'success'];
        }
        update_option("cprocessing_temp_api_key", null);
        return null;
    }

    private function get($url, $api_key = '')
    {
        $headers = $this->set_headers($api_key);

        $response = wp_remote_get($url, array(
                'method' => 'GET',
                'headers' => $headers
            )
        );

        if (is_wp_error($response)) {
            $error_message = $response->get_error_message();
            echo __('Something went wrong', 'cprocessing-bitcoin-payments') . ': ' . $error_message;
        } else {
            return $response;
        }
    }

    private function post($url, $api_key = '', $body = '', $timeout = '')
    {
        $headers = $this->set_headers($api_key);

        if (!empty($body) && isset($body['callback_url'])) {
            $headers .= PHP_EOL . 'Accept: application/json';
        }

        $data = array(
            'method' => 'POST',
            'headers' => $headers,
            'body' => $body
        );

        if ($timeout) {
            $data['timeout'] = $timeout;
        }

        $response = wp_remote_post($url, $data);

        if (is_wp_error($response)) {
            $error_message = $response->get_error_message();
            echo __('Something went wrong', 'cprocessing-bitcoin-payments') . ': ' . $error_message;
        } else {
            return $response;
        }
    }

    private function set_headers($api_key)
    {
        if ($api_key) {
            return 'Authorization: Bearer ' . $api_key;
        } else {
            return '';
        }
    }
    // Runs when the Bcon Global Test Setup button is clicked
    // Returns any errors or false if no errors
    public function testSetup()
    {
        $test_results = array();
        $active_cryptos = $this->getActiveCurrencies();
        foreach ($active_cryptos as $code => $crypto) {
            $test_results[$code] = $this->test_one_crypto($code);
        }
        return $test_results;
    }

    public function test_one_crypto($crypto)
    {
        $response = $this->get_callbacks($crypto);
        $error_str = $this->check_callback_urls_or_set_one($crypto, $response);
        if (!$error_str) {
            //Everything OK ! Test address generation
            $error_str = $this->test_new_address_gen($crypto, $response);
        }
        if ($error_str) {
            return $error_str;
        }
        // No errors
        return false;
    }

    // Returns WC endpoint of order adding the given extra parameters
    public function get_parameterized_wc_url($params = array())
    {
        $order_url = WC()->api_request_url('WC_Gateway_Cprocessing');
        if (is_array($params) && count($params) > 0) {
            foreach ($params as $param_name => $param_value) {
                $order_url = add_query_arg($param_name, $param_value, $order_url);
            }
        }
        return $order_url;
    }

    // Returns url to redirect the user to during checkout
    public function get_order_checkout_url($order_id)
    {
        $active_cryptos = $this->getActiveCurrencies();
        // Check if more than one crypto is activated
        $order_hash = $this->encrypt_hash($order_id);
        if (count($active_cryptos) > 1) {
            $order_url = $this->get_parameterized_wc_url(array('select_crypto' => $order_hash));
        } elseif (count($active_cryptos) === 1) {
            $order_url = $this->get_parameterized_wc_url(array('show_order' => $order_hash, 'crypto' => array_keys($active_cryptos)[0]));
        } else if (count($active_cryptos) === 0) {
            $order_url = $this->get_parameterized_wc_url(array('crypto' => 'empty'));
        }
        return $order_url;
    }

    // Check if a template is a nojs template
    public function is_nojs_template($template_name)
    {
        if (strpos($template_name, 'nojs') === 0) {
            return true;
        }
        return false;
    }

    // Check if the nojs setting is activated
    public function is_nojs_active()
    {
        return get_option('cprocessing_nojs', false);
    }

    // Check if a lite mode setting is activated
    public function is_lite_mode_active()
    {
        return get_option('cprocessing_lite', false);
    }

    public function is_error_template($template_name)
    {
        if (strpos($template_name, 'error') === 0) {
            return true;
        }
        return false;
    }

    // Adds the header to the cprocessing page
    public function load_cprocessing_header($template_name, $additional_script = NULL)
    {

        // Lite mode will render without wordpress theme headers
        if ($this->is_lite_mode_active()) {
            ?>
            <link rel="stylesheet" type="text/css"
                  href="<?php echo plugins_url('css/order.css', dirname(__FILE__)); ?>">
            <?php
            if ($template_name === 'checkout') {
                ?>
                <script src="<?php echo plugins_url('js/vendors/reconnecting-websocket.min.js', dirname(__FILE__)); ?>"
                        defer="defer"></script>
                <script src="<?php echo plugins_url('js/vendors/qrious.min.js', dirname(__FILE__)); ?>"
                        defer="defer"></script>
                <script><?php echo $additional_script; ?></script>
                <script src="<?php echo plugins_url('js/checkout.js', dirname(__FILE__)); ?>" defer="defer"></script>
                <?php
            }
        } else {
            add_action('wp_enqueue_scripts', 'cprocessing_enqueue_stylesheets');

            // wp_enqueue_scripts needs to be called before get_header(), but the scripts are loaded in footer as
            // $in_footer is set to TRUE for scripts in cprocessing_enqueu_scripts

            if ($template_name === 'checkout') {

                add_action('wp_enqueue_scripts', 'cprocessing_enqueue_scripts');

                if (isset($additional_script)) {
                    add_action('wp_enqueue_scripts', function () use ($additional_script) {
                        wp_add_inline_script('cprocessing-checkout', $additional_script, 'before');
                    });
                }
            }

            get_header();
        }
    }

    // Adds the footer to the cprocessing page
    public function load_cprocessing_footer($template_name)
    {

        // Lite mode will render without wordpress theme footers
        if (!$this->is_lite_mode_active()) {
            get_footer();
        }
    }

    public function set_template_context($context)
    {
        // Todo: With WP 5.5+, the load_template methods supports args
        // and can be used as a replacement to this.
        foreach ($context as $key => $value) {
            set_query_var($key, $value);
        }
    }

    // Adds the selected template to the cprocessing page
    public function load_cprocessing_template($template_name, $context = array(), $additional_script = NULL)
    {
        $this->load_cprocessing_header($template_name, $additional_script);

        // Load the selected template
        $template = 'cprocessing_' . $template_name . '.php';

        // Load Template Context
        $this->set_template_context($context);

        // Check if child theme or parent theme have overridden the template
        if ($overridden_template = locate_template($template)) {
            load_template($overridden_template);
        } else {
            load_template(plugin_dir_path(__FILE__) . "../templates/" . $template);
        }

        $this->load_cprocessing_footer($template_name);

        exit();
    }

    public function calculate_order_params($order)
    {
        // Check if order is unused or new
        if ($order['status'] == -1) {
            $wc_order = new WC_Order($order['order_id']);
            $order['value'] = $wc_order->get_total();
            $order['currency'] = get_woocommerce_currency();
            if (get_woocommerce_currency() != 'BTC') {
                $responseObj = $this->get_price($order['currency'], $order['crypto']);
                if ($responseObj->response_code != 200) {
                    exit();
                }
                $price = $responseObj->price;
                $price = $price * 100 / (100 + get_option('cprocessing_margin', 0));
            } else {
                $price = 1;
            }
            $order['satoshi'] = intval(round(1.0e8 * $wc_order->get_total() / $price));
            if($order['crypto'] == 'bnb') $order['satoshi'] = $this->fix_displaying_small_values($order['satoshi']);
        }
        return $order;
    }

    // Save the new address to the WooCommerce order
    public function record_address($order_id, $crypto, $address)
    {
        update_post_meta($order_id, $crypto . '_address', $address);
    }

    public function create_new_order($order_id, $crypto)
    {
        $responseObj = $this->new_address($crypto);
        if ($responseObj->response_code != 200) {
            return array("error" => $responseObj->response_message);
        }

        $address = $responseObj->address;
        $order = array(
            'order_id' => $order_id,
            'status' => -1,
            'crypto' => $crypto,
            'address' => $address,
        );

        $order['memo'] = isset($responseObj->memo) ? $responseObj->memo : '';

        $order = $this->calculate_order_params($order);
        return $order;
    }

    public function get_error_context($error_type)
    {
        $context = array();

        if ($error_type == 'generic') {
            // Show Generic Error to Client.
            $context['error_title'] = __('Could not generate new address (This may be a temporary error. Please try again)', 'cprocessing-bitcoin-payments');
            $context['error_msg'] = __('If this continues, please ask website administrator to do following:<br/><ul><li>Login to admin panel, navigate to Settings > Bcon Global > Currencies and click Test Setup to diagnose the exact issue.</li><li>Check Bcon Global registered email address for error messages</li>', 'cprocessing-bitcoin-payments');
        } else if ($error_type == 'underpaid') {
            $context['error_title'] = '';
            $context['error_msg'] = __('Paid order BTC amount is less than expected. Contact merchant', 'cprocessing-bitcoin-payments');
        }

        return $context;
    }

    public function fix_displaying_small_values($satoshi)
    {
        if ($satoshi < 10000) {
            return rtrim(number_format($satoshi / 1.0e8, 8), 0);
        } else {
            return $satoshi / 1.0e8;
        }
    }

    public function get_checkout_context($order, $crypto){
        $context = array();
        $error_context = NULL;

        $context['order_id'] = $order['order_id'];

        $cryptos = $this->getActiveCurrencies();
        $context['crypto'] = $cryptos[$crypto];

        if (array_key_exists('error', $order)) {
            $error_context = $this->get_error_context('generic');
        } else {
            $context['order'] = $order;

            if ($order['status'] == -2) {
                // Payment is Underpaid
                $error_context = $this->get_error_context('underpaid');
            } elseif ($order['status'] >= 0) {
                // Payment is Received
                $this->redirect_finish_order($order_id);
            } else {
                // Display Checkout Page
                if($order['crypto'] == 'bnb') {
                    $context['order_amount'] = $order['satoshi'];
                } else {
                    $context['order_amount'] = $this->fix_displaying_small_values($order['satoshi']);
                }
                $context['payment_uri'] = $context['crypto']['uri'] . ":" . $order['address'] . "?amount=" . $context['order_amount'];
                //Using svg library qrcode.php to generate QR Code in NoJS mode
                $context['qrcode_svg_element'] = $this->generate_qrcode_svg_element($context['payment_uri']);
            }
        }

        if ($error_context != NULL) {
            $context = array_merge($context, $error_context);
        }

        if(isset($context['order']['memo'])) {
            $context['order']['address'] = str_replace('_' . $context['order']['memo'], '', $context['order']['address']);
            $context['payment_uri'] = str_replace('_' . $context['order']['memo'], '', $context['payment_uri']);
        }

        return $context;
    }

    public function get_checkout_template($context)
    {
        if (array_key_exists('error_msg', $context)) {
            return 'error';
        } else {
            return ($this->is_nojs_active()) ? 'nojs_checkout' : 'checkout';
        }
    }

    public function get_checkout_script($context, $template_name)
    {
        $script = NULL;

        if ($template_name === 'checkout') {
            $script = "const cprocessing_data = '" . json_encode(array(
                    'crypto' => $context['crypto'],
                    'crypto_address' => $context['order']['address'],
                    'time_period' => get_option('cprocessing_timeperiod', 60),
                    'finish_order_url' => $this->get_wc_order_received_url($context['order_id']),
                    'payment_uri' => $context['payment_uri']
                )) . "'";
        }

        return $script;
    }

    // Load the the checkout template in the page
    public function load_checkout_template($order_id, $crypto)
    {
        // Create or update the order
        $order = $this->process_order($order_id, $crypto);
        // Load Checkout Context
        $context = $this->get_checkout_context($order, $crypto);
        // Get Template to Load
        $template_name = $this->get_checkout_template($context);
        // Get any additional inline script to load
        $script = $this->get_checkout_script($context, $template_name);
        // Load the template
        $this->load_cprocessing_template($template_name, $context, $script);
    }

    public function get_wc_order_received_url($order_id)
    {
        $wc_order = new WC_Order($order_id);
        return $wc_order->get_checkout_order_received_url();
    }

    // Redirect the user to the woocommerce finish order page
    public function redirect_finish_order($order_id)
    {
        $wc_order = new WC_Order($order_id);
        wp_safe_redirect($wc_order->get_checkout_order_received_url());
        exit();
    }

    // Fetch the correct crypto order linked to the order id
    public function get_order_by_id_and_crypto($order_id, $crypto)
    {
        global $wpdb;
        $order = $wpdb->get_row(
            $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "cprocessing_orders WHERE order_id = %s AND crypto = %s", array($order_id, $crypto)),
            ARRAY_A
        );
        if ($order) {
            return $order;
        }
        return false;
    }

    // Inserts a new order in cprocessing_orders table
    public function insert_order($order)
    {
        global $wpdb;
        $wpdb->hide_errors();
        $table_name = $wpdb->prefix . 'cprocessing_orders';
        return $wpdb->insert(
            $table_name,
            $order
        );
    }

    // Updates an order in cprocessing_orders table
    public function update_order($order)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'cprocessing_orders';
        $wpdb->replace(
            $table_name,
            $order
        );
    }

    // Check and update the crypto order or create a new order
    public function process_order($order_id, $crypto)
    {
        $order = $this->get_order_by_id_and_crypto($order_id, $crypto);
        if ($order) {
            // Update the existing order info
            $order = $this->calculate_order_params($order);
            $this->update_order($order);
        } else {
            // Create and add the new order to the database
            $order = $this->create_new_order($order_id, $crypto);

            if (array_key_exists("error", $order)) {
                // Some error in Address Generation from API, return the same array.
                return $order;
            }
            if (!$this->insert_order($order)) {
                // insert_order fails if duplicate address found. Ensures no duplicate orders in the database
                return array("error" => __("Duplicate Address Error. This is a Temporary error, please try again", 'cprocessing-bitcoin-payments'));
            }
            $this->record_address($order_id, $crypto, $order['address']);
        }
        return $order;
    }

    // Get the order info by id and crypto
    public function get_order_info($order_id, $crypto)
    {
        $order = $this->process_order($order_id, $crypto);
        header("Content-Type: application/json");
        exit(json_encode($order));
    }

    // Get the order info by crypto address
    public function get_order_by_address($address)
    {
        global $wpdb;
        $order = $wpdb->get_row(
            $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "cprocessing_orders WHERE address = %s ORDER BY order_id DESC", array($address)),
            ARRAY_A
        );
        if ($order) {
            return $order;
        }
        exit(__("Error: Bcon Global order not found", 'cprocessing-bitcoin-payments'));
    }

    // Check if the callback secret in the request matches
    public function check_callback_secret($secret)
    {
        $callback_secret = get_option("cprocessing_callback_secret");
        if ($callback_secret && $callback_secret == $secret) {
            return true;
        }
        exit(__("Error: secret does not match", 'cprocessing-bitcoin-payments'));
    }

    public function save_transaction($value, $order, $wc_order)
    {
        if (!metadata_exists('post', $wc_order->get_id(), 'cprocessing_' . $order['crypto'] . '_txid', $order['txid'])) {
            update_post_meta($wc_order->get_id(), 'cprocessing_' . $order['crypto'] . '_txid', $order['txid']);
            update_post_meta($wc_order->get_id(), 'expected_' . $order['crypto'], number_format($order['satoshi'] / 1.0e8, 8, '.', ','));
        }
    }

    public function update_paid_amount($status, $value, $order, $wc_order){
        $network_confirmations = get_option("cprocessing_network_confirmation",2);
        if ($status >= $network_confirmations && !metadata_exists('post',$wc_order->get_id(),'paid_'. $order['crypto']) )  {
          update_post_meta($wc_order->get_id(), 'paid_'. $order['crypto'], number_format($value/1.0e8, 8, '.', ','));
          $status = $this->check_paid_amount($status, $value, $order, $wc_order);
          $this->update_temp_draw_amount($value);
          return $status;
        }
        return $status;
    }

    // Check for underpayment, overpayment or correct amount
    public function check_paid_amount($status, $value, $order, $wc_order)
    {
        $underpayment_slack = get_option("cprocessing_underpayment_slack", 0) / 100 * $order['satoshi'];
        if ($order['satoshi'] - $underpayment_slack > $value) {
            $status = -2; //Payment error , amount less than expected
            $wc_order->update_status('failed', __('Paid amount less than expected.', 'cprocessing-bitcoin-payments'));
        } else {
            $wc_order->add_order_note(__('Payment completed', 'cprocessing-bitcoin-payments'));
            $wc_order->payment_complete($order['txid']);
        }
        if ($order['satoshi'] < $value) {
            $wc_order->add_order_note(__('Paid amount more than expected.', 'cprocessing-bitcoin-payments'));
        }
        return $status;
    }

    // Keep track of funds in temp wallet
    public function update_temp_draw_amount($value)
    {
        if (get_option('cprocessing_temp_api_key') && !get_option("cprocessing_api_key")) {
            $current_temp_amount = get_option('cprocessing_temp_withdraw_amount');
            $new_temp_amount = $current_temp_amount + $value;
            update_option('cprocessing_temp_withdraw_amount', $new_temp_amount);
        }
    }

    // Process the cprocessing callback
    public function process_callback($secret, $address, $status, $value, $txid, $rbf, $memo)
    {
        $this->check_callback_secret($secret);

        if(isset($memo) && !empty($memo)) {
            $address .= '_' . $memo;
        }

        $order = $this->get_order_by_address($address);
        $wc_order = wc_get_order($order['order_id']);

        if (empty($wc_order)) {
            exit(__("Error: Woocommerce order not found", 'cprocessing-bitcoin-payments'));
        }

        $order['txid'] = $txid;

        if (!$rbf) {
            $this->save_transaction($value, $order, $wc_order);
            $status = $this->update_paid_amount($status, $value, $order, $wc_order);
        }

        $order['status'] = $status;
        $this->update_order($order);
    }

    public function generate_qrcode_svg_element($data)
    {
        include plugin_dir_path(__FILE__) . 'qrcode.php';
        $codeText = sanitize_text_field($data);
        return QRcode::svg($codeText);
    }

    /**
     * Encrypts a string using the application secret. This returns a hex representation of the binary cipher text
     *
     * @param  $input
     * @return string
     */
    public function encrypt_hash($input)
    {
        $encryption_algorithm = 'AES-128-CBC';
        $hashing_algorith = 'sha256';
        $secret = get_option('cprocessing_callback_secret');;
        $key = hash($hashing_algorith, $secret, true);
        $iv = substr($secret, 0, 16);

        $cipherText = openssl_encrypt(
            $input,
            $encryption_algorithm,
            $key,
            OPENSSL_RAW_DATA,
            $iv
        );

        return bin2hex($cipherText);
    }

    /**
     * Decrypts a string using the application secret.
     *
     * @param  $hash
     * @return string
     */
    public function decrypt_hash($hash)
    {
        $encryption_algorithm = 'AES-128-CBC';
        $hashing_algorith = 'sha256';
        $secret = get_option('cprocessing_callback_secret');;
        // prevent decrypt failing when $hash is not hex or has odd length
        if (strlen($hash) % 2 || !ctype_xdigit($hash)) {
            echo __("Error: Incorrect Hash. Hash cannot be validated.", 'cprocessing-bitcoin-payments');
            exit();
        }

        // we'll need the binary cipher
        $binaryInput = hex2bin($hash);
        $iv = substr($secret, 0, 16);
        $cipherText = $binaryInput;
        $key = hash($hashing_algorith, $secret, true);

        $decrypted = openssl_decrypt(
            $cipherText,
            $encryption_algorithm,
            $key,
            OPENSSL_RAW_DATA,
            $iv
        );

        if (empty(wc_get_order($decrypted))) {
            echo __("Error: Incorrect hash. Woocommerce order not found.", 'cprocessing-bitcoin-payments');
            exit();
        }

        return $decrypted;
    }
}
