<?php

/**
 * This class provides the functions needed for extending the WooCommerce
 * Payment Gateway class
 *
 * @class   WC_Gateway_Cprocessing
 * @extends WC_Payment_Gateway
 * @version 2.0.1
 * @author  CryptoProcessing Inc.
 */
class WC_Gateway_Cprocessing extends WC_Payment_Gateway
{
    public function __construct()
    {
        load_plugin_textdomain('cprocessing-bitcoin-payments', false, dirname(plugin_basename(__FILE__)) . '/languages/');

        $this->id   = 'cprocessing';

        $this->has_fields        = false;
        $this->order_button_text = __('Pay with Bcon Global', 'cprocessing-bitcoin-payments');

        $this->init_form_fields();
        $this->init_settings();

        $cprocessing = new CryptoProcessing();

        $this->title       = 'Bcon Global: ';

        $currencies = $cprocessing->getActiveCurrencies();
        $currencyKey = 0;
        foreach ($currencies as $code => $crypto) {
            $code = mb_strtoupper($code);
            if(count($currencies) == $currencyKey+1) {
                $this->title .= ' ' . $code;
            } else {
                $this->title .= ' ' . $code . ',';
            }
            $currencyKey++;
        }

        $this->description = $this->get_option('description');

        // Actions
        add_action(
            'woocommerce_update_options_payment_gateways_' . $this->id, array(
                $this,
                'process_admin_options'
            )
        );
        add_action(
            'woocommerce_receipt_cprocessing', array(
                $this,
                'receipt_page'
            )
        );

        // Payment listener/API hook
        add_action(
            'woocommerce_api_wc_gateway_cprocessing', array(
                $this,
                'handle_requests'
            )
        );
    }

    public function init_form_fields()
    {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Enable Bcon Global plugin', 'cprocessing-bitcoin-payments'),
                'type' => 'checkbox',
                'label' => __('Show bitcoin as an option to customers during checkout?', 'cprocessing-bitcoin-payments'),
                'default' => 'yes'
            ),
            'title' => array(
                'title' => __('Title', 'cprocessing-bitcoin-payments'),
                'type' => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'cprocessing-bitcoin-payments'),
                'default' => __('Bitcoin', 'cprocessing-bitcoin-payments')
            ),
            'description' => array(
                'title' => __( 'Description', 'cprocessing-bitcoin-payments' ),
                'type' => 'text',
                'description' => __('This controls the description which the user sees during checkout.', 'cprocessing-bitcoin-payments'),
                'default' => ''
            )
        );
    }

    public function process_admin_options()
    {
        if (!parent::process_admin_options()) {
            return false;
        }
    }

    // Woocommerce process payment, runs during the checkout
    public function process_payment($order_id)
    {
        include_once 'CryptoProcessing';
        $cprocessing = new CryptoProcessing;
        $order_url = $cprocessing->get_order_checkout_url($order_id);

        return array(
            'result'   => 'success',
            'redirect' => $order_url
        );
    }

    // Handles requests to the cprocessing page
    // Sanitizes all request/input data
    public function handle_requests()
    {
        $show_order = isset($_GET["show_order"]) ? sanitize_text_field(wp_unslash($_GET['show_order'])) : "";
        $crypto = isset($_GET["crypto"]) ? sanitize_key($_GET['crypto']) : "";
        $select_crypto = isset($_GET["select_crypto"]) ? sanitize_text_field(wp_unslash($_GET['select_crypto'])) : "";
        $finish_order = isset($_GET["finish_order"]) ? sanitize_text_field(wp_unslash($_GET['finish_order'])) : "";
        $get_order = isset($_GET['get_order']) ? sanitize_text_field(wp_unslash($_GET['get_order'])) : "";
        $secret = isset($_GET['secret']) ? sanitize_text_field(wp_unslash($_GET['secret'])) : "";
        $addr = isset($_GET['addr']) ? sanitize_text_field(wp_unslash($_GET['addr'])) : "";
        $status = isset($_GET['status']) ? intval($_GET['status']) : "";
        $value = isset($_GET['value']) ? $_GET['value'] : "";
        $memo = isset($_GET['memo']) ? $_GET['memo'] : "";
        $txid = isset($_GET['txid']) ? sanitize_text_field(wp_unslash($_GET['txid'])) : "";
        $rbf = isset($_GET['rbf']) ? wp_validate_boolean(intval(wp_unslash($_GET['rbf']))) : "";

        include_once 'CryptoProcessing.php';
        $cprocessing = new CryptoProcessing;

        if($crypto === "empty"){
            $cprocessing->load_cprocessing_template('no_crypto_selected');
        }else if ($show_order && $crypto) {
            $order_id = $cprocessing->decrypt_hash($show_order);
            $cprocessing->load_checkout_template($order_id, $crypto);
        }else if ($select_crypto) {
            $cprocessing->load_cprocessing_template('crypto_options');
        }else if ($finish_order) {
            $order_id = $cprocessing->decrypt_hash($finish_order);
            $cprocessing->redirect_finish_order($order_id);
        }else if ($get_order && $crypto) {
            $order_id = $cprocessing->decrypt_hash($get_order);
            $cprocessing->get_order_info($order_id, $crypto);
        }else if ($secret && $addr && isset($status) && $value && $txid) {
            $cprocessing->process_callback($secret, $addr, $status, $value, $txid, $rbf, $memo);
        }

        exit();
    }
}
