<?php
    /**
     * Crypto-Processing Checkout Page (JS Enabled)
     *
     * The following variables are available to be used in the template along with all WP Functions/Methods/Globals
     *
     * $order: Order Object
     * $order_id: WooCommerce Order ID
     * $order_amount: Crypto Amount
     * $crypto: Crypto Object (code, name, uri) e.g. (btc, Bitcoin, bitcoin)
     * $payment_uri: Crypto URI with Amount and Protocol
     * $qrcode_svg_element: Generate QR Code when NoJS mode is active.
     */
?>
<div id="cprocessing_checkout">
    <div class="cprocessing-order-container">
        <!-- Heading row -->
        <div class="cprocessing-order-heading">
            <div class="cprocessing-order-heading-wrapper">
                <div class="cprocessing-order-id">
                    <span class="cprocessing-order-number"><?=__('Order #', 'cprocessing-bitcoin-payments')?><?php echo $order_id; ?></span>
                </div>
            </div>
        </div>

        <!-- Spinner -->
        <div class="cprocessing-spinner-wrapper">
            <div class="cprocessing-spinner"></div>
        </div>

        <!-- Display Error -->
        <div class="cprocessing-display-error">
            <h2><?=__('Display Error', 'cprocessing-bitcoin-payments')?></h2>
            <p><?=__('Unable to render correctly, Note to Administrator: Please try enabling other modes like No Javascript or Lite mode in the Bcon Global plugin > Advanced Settings.', 'cprocessing-bitcoin-payments')?></p>
        </div>

        <!-- Payment Expired -->
        <div class="cprocessing-order-expired-wrapper">
            <h3 class="warning cprocessing-status-warning"><?=__('Payment Expired', 'cprocessing-bitcoin-payments')?></h3><br/>
            <p><a href="#" id="cprocessing-try-again"><?=__('Click here to try again', 'cprocessing-bitcoin-payments')?></a></p>
        </div>

        <!-- Checkout Panel -->
        <div class="cprocessing-order-panel">
            <div class="cprocessing-order-info">
                <div class="cprocessing-bitcoin-pane">
                    <div class="cprocessing-btc-info">
                        <!-- Left Side -->
                        <div class="cprocessing-qr-code">
                            <!-- QR and Open in wallet -->
                            <div class="cprocessing-qr">
                                <a href="<?php echo $payment_uri; ?>" target="_blank">
                                    <canvas id="cprocessing-qr-code"></canvas>
                                </a>
                            </div>
                            <div class="cprocessing-qr-code-hint">
                                <a href="<?php echo $payment_uri; ?>" target="_blank"><?=__('Open in wallet', 'cprocessing-bitcoin-payments')?></a>
                            </div>
                        </div>

                        <!-- Right Side -->
                        <div class="cprocessing-amount">
                            <div class="cprocessing-bg">
                                <!-- Order Amounts -->
                                <?php if(!empty($order['memo'])) : ?>
                                    <b>Mention this MEMO when paying: <?php echo $order['memo']; ?></b>
                                <?php endif; ?>
                                <div class="cprocessing-amount">
                                    <div class="cprocessing-amount-text"><?=__('To pay, send exactly this', 'cprocessing-bitcoin-payments')?> <?=strtoupper($crypto['code'])?></div>
                                    <div class="cprocessing-copy-amount-text"><?=__('Copied to clipboard', 'cprocessing-bitcoin-payments')?></div>
                                    <ul id="cprocessing-amount-input" class="cprocessing-amount-input">
                                        <li id="cprocessing-amount-copy"><?php echo $order_amount; ?></li>
                                        <li><?php echo strtoupper($crypto['code']); ?></li>
                                        <li class="cprocessing-grey"> ≈ </li>
                                        <li class="cprocessing-grey"><?php echo $order['value']; ?></li>
                                        <li class="cprocessing-grey"><?php echo $order['currency']; ?></li>
                                    </ul>
                                </div>
                                <!-- Order Address -->
                                <div class="cprocessing-address">
                                    <div class="cprocessing-address-text"><?=__('To this '.strtolower($crypto['name']).' address', 'cprocessing-bitcoin-payments')?></div>
                                    <div class="cprocessing-copy-address-text"><?=__('Copied to clipboard', 'cprocessing-bitcoin-payments')?></div>
                                    <ul id="cprocessing-address-input" class="cprocessing-address-input">
                                        <li id="cprocessing-address-copy"><?php echo $order['address']; ?></li>
                                    </ul>
                                </div>
                                <!-- Order Countdown Timer -->
                                <div class="cprocessing-progress-bar-wrapper">
                                    <div class="cprocessing-progress-bar-container">
                                    <div class="cprocessing-progress-bar" style="width: 0%;"></div>
                                </div>
                            </div>

                            <span class="cprocessing-time-left">00:00 min</span>
                        </div>
                    </div>
                </div>
            </div>

        </div>
    </div>
</div>
